#!/bin/bash

# Copyright (C) since 2014 NTT DATA Corporation
#
# This program is free software; you can redistribute it and/or
# Modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation, version 2.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details

# bash configuration
SCRIPT_DIR=$(cd $(dirname $0);pwd)
PROG=`basename $0`

########################################
#  Local Variable
########################################

# script version
SCRIPT_VERSION="1.0.1"

# target account and aws cli profile name

# aws account user id
AWSACCOUNT=""

# profile name
PROFILE="default"

# lock file
LOCK=""

# log directory
LOG_ROOT_DIR=/var/log/aws/cloudtrail

# temporally directory
TMP_ROOT_DIR=/var/log/aws/cloudtrail/.tmp

# download file directory
LOG_EXPORT_DIR=/var/log/aws/cloudtrail_export

# target region list
TARGET_REGION_LIST="${SCRIPT_DIR}/hinemos_cloudtrail_logcheck.list"

# json cnvert module
LOG_JSON_PARSER="${SCRIPT_DIR}/HinemosCloudTrailLog.py"


########################################
# Function
########################################
#
# Message Logging
#
function Logging() {
	[ "x${QUIET}" != "xtrue" ] && echo "$1"
	/usr/bin/logger "${PROG} $1"
}

#
# show usage
#
function usage {
	echo "usage : ${PROG} -p profile -q [--version] [--help]"
	echo "options:"
	echo "  -p profile  set a aws cli profile name"
	echo "  -q          quiet mode"
	echo "  --version   show version"
	echo "  --help      show usage"
}

#
# Check Local Directory
#
function check_local_directory {

	while [ "$1" != "" ]
	do
		if [ ! -e ${1} ]; then
			mkdir -p ${1}
		fi
		shift
	done

	return 0
}


#
# Check S3 Folder
#
function check_s3_folder {
	ROWS=`aws --profile ${PROFILE} s3 ls ${1} | wc -l `

	if [ ${ROWS} -eq 0 ]; then
		Logging "S3FOLDER does not exist. S3FOLDER=${1}"
		return 1
	fi

	return 0
}

#
# Download CloudTrail Log
#
function download_cloudtrail_logs {
	MSG=`aws --profile ${PROFILE} s3 sync $1 $2 > ${TMPLIST}`
	RET=$?
	if [ ${RET} -ne 0 ]; then
		Logging "${MSG}"
		Logging "Faild to download logs from S3."
		return ${RET}
	fi
  
	COUNT=`cat ${TMPLIST} | wc -l`
	if [ ${COUNT} -eq 0 ]; then
		Logging "Download Files is none."
	else
		tr -d \\r < ${TMPLIST} > ${DOWNLOADLIST}
	fi

	return ${RET}
}


########################################
# SHELL
########################################

# help chekc
for OPT in $@
do
	case $OPT in
		--version)
			echo ${SCRIPT_VERSION}
			exit 0
			;;
		--help)
			usage
			exit 0
			;;
	esac
done

# check parameters
while getopts p:q OPT
do
	case $OPT in
		p)
			PROFILE=$OPTARG
			;;
		q)
			QUIET=true
			;;
		*)
			usage
			exit 1
			;;
	esac
done

shift $((OPTIND - 1))

if [ "x${PROFILE}" == "x" ];then
	echo "option (-p) is not specified."
	usage
	exit 1
fi
Logging "target profile ${PROFILE}"

# target list file check
if [ ! -e ${TARGET_REGION_LIST} ]; then
	Logging "${TARGET_REGION_LIST} is not found."
	exit 1
fi

# get userid
AWSACCOUNT=`aws --profile ${PROFILE} sts get-caller-identity | jq '.Account' | sed s/\"//g`
RET=$?
if [ ! "x${RET}" = "x0" ]; then
	Logging "Connection Failed. PROFILE=${PROFILE}."
	exit ${RET}
fi
Logging "aws account user id for profile ${PROFILE} is ${AWSACCOUNT}"

# date settings
YYYY_T=`date -u +"%Y"`
MM_T=`date -u +"%m"`
DD_T=`date -u +"%d"`
YYYY_Y=`date -u --date '1 day ago' +"%Y"`
MM_Y=`date -u --date '1 day ago' +"%m"`
DD_Y=`date -u --date '1 day ago' +"%d"`

# set output directory
LOG_DIR=${LOG_ROOT_DIR}/${PROFILE}
TMP_DIR=${TMP_ROOT_DIR}/${PROFILE}
check_local_directory ${LOG_DIR} ${TMP_DIR}

# temporary list file
TMPLIST=${TMP_DIR}/download_cloudtraillogs.list
DOWNLOADLIST=${TMP_DIR}/download_cloudtraillogsnew.list
LOCK=${TMP_DIR}/${PROFILE}.lock

# main
Logging "update cloudtrail log. ( profile ${PROFILE} )"

# lock file check
if [ -e ${LOCK} ]; then
	Logging "another process is already running.( profile ${PROFILE} ). this process terminated with exit code 1."
	exit 1
fi
touch ${LOCK}

awk '$0 !~ /^#/{print $0}' ${TARGET_REGION_LIST} | while read line
do
	REGION=$line
	Logging "----"
	Logging "update cloudtrail log. target region ${REGION}"
	Logging "update logfile ${LOG_DIR}/cloudtrail_${REGION}.log"

	# connec test
	aws --profile ${PROFILE} --region ${REGION} cloudtrail describe-trails > /dev/null
	RET=$?
	if [ ! "x${RET}" = "x0" ]; then
		Logging "Connection Failed."
		continue
	fi

	S3BUCKET=`aws --profile ${PROFILE} --region ${REGION} cloudtrail describe-trails | jq '.trailList | sort_by(.Name) | .[0].S3BucketName' | sed s/\"//g`
	Logging "S3BUCKET=${S3BUCKET}."

	S3KeyPrefix=`aws --profile ${PROFILE} --region ${REGION} cloudtrail describe-trails | jq '.trailList | sort_by(.Name) | .[0].S3KeyPrefix' | sed s/\"//g`
	Logging "S3KeyPrefix=${S3KeyPrefix}."

	DOWNLOADPATH_T="${LOG_EXPORT_DIR}/${PROFILE}/${REGION}/${YYYY_T}/${MM_T}/${DD_T}/"
	DOWNLOADPATH_Y="${LOG_EXPORT_DIR}/${PROFILE}/${REGION}/${YYYY_Y}/${MM_Y}/${DD_Y}/"

	if [ "${S3KeyPrefix}" = "null" ]; then
		S3FOLDERPATH_T="s3://${S3BUCKET}/AWSLogs/${AWSACCOUNT}/CloudTrail/${REGION}/${YYYY_T}/${MM_T}/${DD_T}/"
		S3FOLDERPATH_Y="s3://${S3BUCKET}/AWSLogs/${AWSACCOUNT}/CloudTrail/${REGION}/${YYYY_Y}/${MM_Y}/${DD_Y}/"
	else
		S3FOLDERPATH_T="s3://${S3BUCKET}/${S3KeyPrefix}/AWSLogs/${AWSACCOUNT}/CloudTrail/${REGION}/${YYYY_T}/${MM_T}/${DD_T}/"
		S3FOLDERPATH_Y="s3://${S3BUCKET}/${S3KeyPrefix}/AWSLogs/${AWSACCOUNT}/CloudTrail/${REGION}/${YYYY_Y}/${MM_Y}/${DD_Y}/"
	fi 

	Logging "check_local_directory ${DOWNLOADPATH_T}"
	check_local_directory ${DOWNLOADPATH_T}

	Logging "check_s3_folder ${S3FOLDERPATH_T}"
	check_s3_folder ${S3FOLDERPATH_T} 
	RET=$?
	if [ ${RET} -eq 1 ]; then
		Logging "check_s3_folder ${S3FOLDERPATH_Y}"
		check_s3_folder ${S3FOLDERPATH_Y}
		RET=$?
		if [ ${RET} -eq 1 ]; then
			continue
		else
			check_local_directory ${DOWNLOADPATH_Y}
			download_cloudtrail_logs ${S3FOLDERPATH_Y} ${DOWNLOADPATH_Y}
			RET=$?
			if [ ${RET} -ne 0 ]; then
				continue
			else
				DOWNLOADPATH=${DOWNLOADPATH_Y}
			fi
		fi
	else
		download_cloudtrail_logs ${S3FOLDERPATH_T} ${DOWNLOADPATH_T}
		RET=$?
		if [ ${RET} -ne 0 ]; then
			continue
		else
			DOWNLOADPATH=${DOWNLOADPATH_T}
		fi
	fi


	#decompress file & parse
	if [ -e ${DOWNLOADLIST} ];then
		while read line
		do
			FILENAME=${line##*/}
			FILENAME_WITHOUT_EXTENSION=${FILENAME%.*}

			if [ -e ${DOWNLOADPATH}${FILENAME} ]; then 
				Logging "update ${FILENAME}."
				cp -p ${DOWNLOADPATH}${FILENAME} ${TMP_DIR}/${FILENAME} && gzip -d ${TMP_DIR}/${FILENAME}
				RET=$?
				if [ ${RET} -ne 0 ]; then
					Logging "Failed to cp or gzip : ${FILENAME}"
					exit ${RET}
				fi

				# convert json log
				#Logging "convert ${FILENAME_WITHOUT_EXTENSION} to ${TMP_DIR}/${FILENAME_WITHOUT_EXTENSION}.log"
				${LOG_JSON_PARSER} -i ${TMP_DIR}/${FILENAME_WITHOUT_EXTENSION} -o ${TMP_DIR}/${FILENAME_WITHOUT_EXTENSION}.log

				# redirect log
				#Logging "redirect ${FILENAME_WITHOUT_EXTENSION}.log to ${LOG_DIR}/cloudtrail_${REGION}.log"
				cat ${TMP_DIR}/${FILENAME_WITHOUT_EXTENSION}.log >> ${LOG_DIR}/cloudtrail_${REGION}.log
				rm -f ${TMP_DIR}/${FILENAME_WITHOUT_EXTENSION}.log
				rm -f ${TMP_DIR}/${FILENAME_WITHOUT_EXTENSION}

				RET=$?
				if [ ${RET} -ne 0 ]; then
					Logging   "Failed to Parse by ${LOG_JSON_PARSER} : ${FILENAME}"
					break
				else
					rm -f ${LOG_DIR}/${FILENAME_WITHOUT_EXTENSION}
				fi
			fi
		done < ${DOWNLOADLIST}
	fi

	#remove tmpfile
	if [ -e ${TMPLIST} ];then
		rm -f ${TMPLIST}
	fi

	if [ -e ${DOWNLOADLIST} ];then
		rm -f ${DOWNLOADLIST}
	fi

	Logging "successful update cloudtrail log."
	Logging "----"
done

########################################
# Termination Processing
########################################

Logging "successful update cloudtrail log. ( profile ${PROFILE} )"
rm -Rf ${LOCK}
exit ${RET}


